# Copyright 2022 Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)

from odoo import models


class SaleOrderLine(models.Model):
    _inherit = "sale.order.line"

    # ------------------------------------------------------
    # Business method
    # ------------------------------------------------------
    def _prepare_invoice_line(self, **optional_values):
        # Method to update account on invoice line in case service account
        # is different from base account (AP32 has 2 different accounts for the same product)
        res = super(SaleOrderLine, self)._prepare_invoice_line(**optional_values)
        if res.get("product_id"):
            product_id = self.env["product.product"].browse(res.get("product_id"))
            product_tmpl_id = product_id.product_tmpl_id
            if product_tmpl_id.property_account_service_income_id and (
                product_tmpl_id.property_account_income_id.id
                != product_tmpl_id.property_account_service_income_id.id
            ):
                furniture_categ_id = self.env.ref(
                    "ap_sale_project.product_category_plantation_supply"
                )
                vat_furniture_categ = (
                    self.env["product.category"].browse(furniture_categ_id.id).taxes_id
                )
                if res.get("tax_ids") == [(6, 0, vat_furniture_categ.mapped("id"))]:
                    res[
                        "account_id"
                    ] = product_tmpl_id.property_account_service_income_id.id
        return res
