# Copyright 2021 Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)

from odoo import _, models
from odoo.exceptions import UserError


class SaleProject(models.Model):
    _inherit = "sale.project"

    def update_order_lines(self):
        """
        Rewrite function to use Taxes
        """
        plant_categ_id = self.env.ref("ap_sale_project.product_category_plant")
        vat_plant_categ = (
            self.env["product.category"].browse(plant_categ_id.id).taxes_id
        )
        furniture_categ_id = self.env.ref(
            "ap_sale_project.product_category_plantation_supply"
        )
        vat_furniture_categ = (
            self.env["product.category"].browse(furniture_categ_id.id).taxes_id
        )
        lines = self.env["sale.intervention"].read_group(
            (("project_id", "=", self.id),),
            ("price", "intervention_length", "plant_qty"),
            "intervention_type_id",
            lazy=False,
        )
        for line in lines:
            product_tmpl_id = line["intervention_type_id"][0]
            product_tmpl = self.env["product.template"].browse(product_tmpl_id)
            pricelist = self.project_subvention_id.product_pricelist_ids.filtered(
                lambda i: i.intervention_type_id.id == product_tmpl_id
            )
            if (
                pricelist
                and pricelist.item_ids.filtered(lambda i: i.categ_id == plant_categ_id)
                and self.env.user.company_id.vat
            ):
                quantity = (
                    line["intervention_length"]
                    if product_tmpl.uom_name == "m"
                    else line["plant_qty"]
                )
                if quantity != 0:
                    price_unit = line["price"] / quantity
                    plant_pricelist_item = pricelist.item_ids.filtered(
                        lambda i: i.categ_id == plant_categ_id
                    )
                    plant_price = plant_pricelist_item.fixed_price
                    existing_lines = self.sale_order_id.order_line.filtered(
                        lambda r: r.product_id.id == product_tmpl.product_variant_id.id
                        and r.sale_project_id.id == self.id
                    )
                    if existing_lines:
                        sol_plant = existing_lines.filtered(
                            lambda sol: sol.tax_id == vat_plant_categ
                        )
                        if len(sol_plant) == 1:
                            sol_plant.write(
                                {
                                    "product_uom_qty": quantity,
                                }
                            )
                        else:
                            raise UserError(
                                _(
                                    "Impossible de mettre à jour la ligne des "
                                    "plants dans le devis. "
                                )
                            )
                        sol_furniture = existing_lines.filtered(
                            lambda sol: sol.tax_id == vat_furniture_categ
                        )
                        if len(sol_furniture) == 1:
                            sol_furniture.write(
                                {
                                    "product_uom_qty": quantity,
                                    "price_unit": price_unit - plant_price,
                                }
                            )
                        else:
                            raise UserError(
                                _(
                                    "Impossible de mettre à jour la ligne des "
                                    "fournitures dans le devis. "
                                )
                            )
                    else:
                        sols = list()
                        data_sol_section = {
                            "order_id": self.sale_order_id.id,
                            "product_id": product_tmpl.product_variant_id.id,
                            "product_uom_qty": quantity,
                            "sale_project_id": self.id,
                        }
                        sol_name = "Projet " + self.name + " - " + product_tmpl.name
                        sols.append(
                            {
                                **data_sol_section,
                                **{
                                    "name": sol_name,
                                    "display_type": "line_section",
                                },
                            }
                        )
                        sols.append(
                            {
                                **data_sol_section,
                                **{
                                    "price_unit": plant_price,
                                    "name": "Plants garantis 3 ans",
                                    "tax_id": [(6, 0, vat_plant_categ.ids)],
                                },
                            }
                        )
                        sols.append(
                            {
                                **data_sol_section,
                                **{
                                    "price_unit": price_unit - plant_price,
                                    "name": "Paillage, protections, "
                                    "ingénierie, livraison et suivi",
                                    "tax_id": [(6, 0, vat_furniture_categ.ids)],
                                },
                            }
                        )
                        for sol in sols:
                            self.env["sale.order.line"].with_context(default_project_id=False).create(sol)
            else:
                super(SaleProject, self).update_order_lines()
        self.sale_order_id.clean_sale_order()
